/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.configuration.metatype.annotations;

import aQute.bnd.annotation.xml.XMLAttribute;

import com.liferay.petra.string.StringBundler;
import com.liferay.petra.string.StringPool;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * @author Iván Zaera
 */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.TYPE)
@XMLAttribute(
	embedIn = "*", namespace = ExtendedObjectClassDefinition.XML_NAMESPACE,
	prefix = ExtendedObjectClassDefinition.XML_ATTRIBUTE_PREFIX
)
public @interface ExtendedObjectClassDefinition {

	public static final String XML_ATTRIBUTE_PREFIX = "cf";

	public static final String XML_NAMESPACE =
		"http://www.liferay.com/xsd/liferay-configuration-admin_1_0_0.xsd";

	public String category() default "";

	public String[] descriptionArguments() default {};

	public String factoryInstanceLabelAttribute() default "";

	public boolean generateUI() default true;

	public String liferayLearnMessageKey() default "";

	public String liferayLearnMessageResource() default "";

	public String[] nameArguments() default {};

	public Scope scope() default Scope.SYSTEM;

	public String settingsId() default "";

	/**
	 * Whether or not the configuration's visibility will be limited to the
	 * declared scope.
	 *
	 * If true, the configuration will only be visible at the declared scope.
	 *
	 * If false, the configuration will be visible at the declared scope and all
	 * broader scopes, unless otherwise restricted by a
	 * ConfigurationVisibilityController.
	 *
	 * The default value is false.
	 *
	 * @review
	 */
	public boolean strictScope() default false;

	public enum Scope {

		COMPANY("companyId", "company"), GROUP("groupId", "group"),
		PORTLET_INSTANCE("portletInstanceId", "portlet-instance"),
		SYSTEM(null, "system");

		public boolean equals(String value) {
			return _value.equals(value);
		}

		public String getDelimiterString() {
			return StringBundler.concat(_SEPARATOR, name(), _SEPARATOR);
		}

		public String getPropertyKey() {
			return _propertyKey;
		}

		public String getValue() {
			return _value;
		}

		@Override
		public String toString() {
			return _value;
		}

		private Scope(String propertyKey, String value) {
			_propertyKey = propertyKey;
			_value = value;
		}

		private static final String _SEPARATOR = StringPool.DOUBLE_UNDERLINE;

		private final String _propertyKey;
		private final String _value;

	}

}